@@ -4,7 +4,7 @@
 #
 # Author: Slaven Rezic
 #
-# Copyright (C) 2017,2018 Slaven Rezic. All rights reserved.
+# Copyright (C) 2017,2018,2019 Slaven Rezic. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -17,7 +17,7 @@ use warnings;
 
 {
     package Doit;
-    our $VERSION = '0.025_53';
+    our $VERSION = '0.025_54';
 
     use constant IS_WIN => $^O eq 'MSWin32';
 }
@@ -390,7 +390,11 @@ use warnings;
     }
 
     sub cmd_chmod {
-	my($self, $mode, @files) = @_;
+	my($self, @args) = @_;
+	my %options; if (@args && ref $args[0] eq 'HASH') { %options = %{ shift @args } }
+	my $quiet = delete $options{quiet};
+	error "Unhandled options: " . join(" ", %options) if %options;
+	my($mode, @files) = @args;
 	my @files_to_change;
 	for my $file (@files) {
 	    my @s = stat($file);
@@ -416,7 +420,7 @@ use warnings;
 				     }
 				 }
 			     },
-			     msg  => sprintf("chmod 0%o %s", $mode, join(" ", @files_to_change)), # shellquote?
+			     ($quiet ? () : (msg => sprintf("chmod 0%o %s", $mode, join(" ", @files_to_change)))), # shellquote?
 			     rv   => scalar @files_to_change,
 			    };
 	    Doit::Commands->new(@commands);
@@ -426,7 +430,11 @@ use warnings;
     }
 
     sub cmd_chown {
-	my($self, $uid, $gid, @files) = @_;
+	my($self, @args) = @_;
+	my %options; if (@args && ref $args[0] eq 'HASH') { %options = %{ shift @args } }
+	my $quiet = delete $options{quiet};
+	error "Unhandled options: " . join(" ", %options) if %options;
+	my($uid, $gid, @files) = @args;
 
 	if (!defined $uid) {
 	    $uid = -1;
@@ -483,7 +491,7 @@ use warnings;
 				     }
 				 }
 			     },
-			     msg  => "chown $uid, $gid, @files_to_change", # shellquote?
+			     ($quiet ? () : (msg => "chown $uid, $gid, @files_to_change")), # shellquote?
 			     rv   => scalar @files_to_change,
 			    };
 	    Doit::Commands->new(@commands);
@@ -844,6 +852,11 @@ use warnings;
 	    }
 
 	    waitpid $pid, 0;
+
+	    if ($errref) {
+		$$errref = $buf{$chld_err};
+	    }
+
 	    if ($statusref) {
 		%$statusref = ( _analyze_dollar_questionmark );
 	    } else {
@@ -852,10 +865,6 @@ use warnings;
 		}
 	    }
 
-	    if ($errref) {
-		$$errref = $buf{$chld_err};
-	    }
-
 	    $buf{$chld_out};
 	};
 
@@ -2088,7 +2097,7 @@ use warnings;
 	require File::Basename;
 	require Net::OpenSSH;
 	require FindBin;
-	my($class, $host, %opts) = @_;
+	my($class, $ssh_or_host, %opts) = @_;
 	my $dry_run = delete $opts{dry_run};
 	my @components = @{ delete $opts{components} || [] };
 	my $debug = delete $opts{debug};
@@ -2103,9 +2112,13 @@ use warnings;
 	$put_to_remote =~ m{^(rsync_put|scp_put)$}
 	    or error "Valid values for put_to_remote: rsync_put or scp_put";
 	my $perl = delete $opts{perl} || 'perl';
+	my $umask = delete $opts{umask};
+	if (defined $umask && $umask !~ m{^\d+$}) {
+	    error "The umask '$umask' does not look correct, it should be a (possibly octal) number";
+	}
 	error "Unhandled options: " . join(" ", %opts) if %opts;
 
-	my $self = bless { host => $host, debug => $debug }, $class;
+	my $self = bless { debug => $debug }, $class;
 	my %ssh_run_opts = (
 	    ($forward_agent ? (forward_agent => $forward_agent) : ()),
 	    ($tty           ? (tty           => $tty)           : ()),
@@ -2114,10 +2127,19 @@ use warnings;
 	    ($forward_agent ? (forward_agent => $forward_agent) : ()),
 	    ($master_opts   ? (master_opts   => $master_opts)   : ()),
 	);
-	my $ssh = Net::OpenSSH->new($host, %ssh_new_opts);
-	$ssh->error
-	    and error "Connection error to $host: " . $ssh->error;
+
+	my($host, $ssh);
+	if (UNIVERSAL::isa($ssh_or_host, 'Net::OpenSSH')) {
+	    $ssh = $ssh_or_host;
+	    $host = $ssh->get_host; # XXX what about username/port/...?
+	} else {
+	    $host = $ssh_or_host;
+	    $ssh = Net::OpenSSH->new($host, %ssh_new_opts);
+	    $ssh->error
+		and error "Connection error to $host: " . $ssh->error;
+	}
 	$self->{ssh} = $ssh;
+
 	{
 	    my $remote_cmd;
 	    if ($dest_os eq 'MSWin32') {
@@ -2203,6 +2225,7 @@ use warnings;
 	    @cmd_worker =
 	    (
 	     @cmd, $perl, "-I.doit", "-I.doit/lib", "-e",
+	     (defined $umask ? qq{umask $umask; } : q{}) .
 	     Doit::_ScriptTools::self_require($FindBin::RealScript) .
 	     q{my $d = Doit->init; } .
 	     Doit::_ScriptTools::add_components(@components) .
